{% extends 'crudlfap/base.html' %}

{% block style_bundle %}
    {{ super() }}
    <link href="{{ static('ryz_ex/ryzom_ex.css') }}" rel="stylesheet">
{% endblock %}

{% block script_bundle %}
    {{ super() }}
    <script src="{{ static('ryz_ex/ryzom_ex.js') }}" defer></script>
{% endblock %}
